import { FC, ReactNode } from 'react';
/**
 * An activity indicator which should be unconditionally rendered because it
 * knows how to hide itself when there's no background activity going on.
 * This is useful for loading indicators that should have exit animations, as
 * the component will stay mounted through the animation.
 */
declare type SelfHidingIndicator = (isLoading: boolean) => ReactNode;
/**
 * The `ActivityIndicator` component provides a way to show some UI when there
 * are background data updates in progress. For example, this would be used to
 * render a thin indeterminate progress bar under a site's top nav.
 *
 * This component wraps around the components for which it should show an
 * indicator. It's common to have an instance near the root of the app, with
 * a handful of other uses for overlay or other similar components.
 */
export declare const ActivityIndicator: FC<{
    /**
     * This can be a `ReactNode` which the monitor will automatically mount and
     * unmount based on whether or not activity is occurring, or a function
     * which takes whether or not the indicator should be visible.
     *
     * The provided node will be rendered as an immediate sibling of the
     * component children; for displaying visually distant indicators, a portal
     * is recommended.
     *
     * If this is omitted, no visual will be displayed. This can be used in
     * conjunction with `useActivityIndicatorStatus` to render the visual
     * indicator as a descendant of this component, rather than an immediate
     * child.
     */
    indicator?: ReactNode | SelfHidingIndicator;
    children?: ReactNode;
}>;
/**
 * Register a data dependency with the nearest `ActivityIndicator` parent to show
 * an indicator when there is stale data on display or a background operation in
 * progress.
 *
 * @param hasActivity Whether or not the hook caller has stale data (e.g. the
 * presented data isn't the latest data) or is performing some background task.
 * @param timeoutBeforeShow Amount of time in ms that `hasActivity` must be
 * `true` uninterrupted before it will ask the context to render the indicator.
 * This defaults to 250ms.
 */
export declare const useActivityIndicator: (hasActivity: boolean, timeoutBeforeShow?: number) => void;
/**
 * Get whether or not the ancestor `ActivityIndicator` is currently indicating.
 * This can be used to insert the visual indicator element as a descendant of
 * the context provider, rather than an immediate child.
 */
export declare const useActivityIndicatorStatus: () => boolean;
export {};
